@@ -17,16 +17,19 @@
 
 import random
 import sys
+import time
+import warnings
 
 from eventlet import event
 from eventlet import greenthread
+from eventlet import timeout as eventlettimeout
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import reflection
 from oslo_utils import timeutils
 import six
 
-from oslo_service._i18n import _LE, _LW, _
+from oslo_service._i18n import _
 
 LOG = logging.getLogger(__name__)
 
@@ -66,7 +69,7 @@ def _safe_wrapper(f, kind, func_name):
         except LoopingCallDone:
             raise  # let the outer handler process this
         except Exception:
-            LOG.error(_LE('%(kind)s %(func_name)r failed'),
+            LOG.error('%(kind)s %(func_name)r failed',
                       {'kind': kind, 'func_name': func_name},
                       exc_info=True)
             return 0
@@ -74,6 +77,47 @@ def _safe_wrapper(f, kind, func_name):
     return func
 
 
+class _Event(object):
+    """A class that provides consistent eventlet/threading Event API.
+
+    It's a copy from the oslo_utils repository, as we need the private version
+    because we don't want to use the threading.Event version.
+    """
+    def __init__(self, *args, **kwargs):
+        self.clear()
+
+    def clear(self):
+        self._set = False
+        self._event = event.Event()
+
+    def is_set(self):
+        return self._set
+
+    def set(self):
+        self._set = True
+        self._event.send(True)
+
+    def wait(self, timeout=None):
+        with eventlettimeout.Timeout(timeout, False):
+            self._event.wait()
+        return self.is_set()
+
+    def is_running(self):
+        warnings.warn("This private API is removed in the next minor release.",
+                      DeprecationWarning)
+        return not self.is_set()
+
+    def done(self):
+        warnings.warn("This private API is removed in the next minor release.",
+                      DeprecationWarning)
+        return self.set()
+
+
+# For backwards compatibility with tests in consumers that were relying on the
+# private interface.
+_ThreadingEvent = _Event
+
+
 class LoopingCallBase(object):
     _KIND = _("Unknown looping call")
 
@@ -84,19 +128,26 @@ class LoopingCallBase(object):
         self.args = args
         self.kw = kw
         self.f = f
-        self._running = False
         self._thread = None
         self.done = None
+        self._abort = _Event()
+
+    @property
+    def _running(self):
+        return not self._abort.is_set()
 
     def stop(self):
-        self._running = False
+        if self._running:
+            self._abort.set()
 
     def wait(self):
         return self.done.wait()
 
     def _on_done(self, gt, *args, **kwargs):
         self._thread = None
-        self._running = False
+
+    def _sleep(self, timeout):
+        self._abort.wait(timeout)
 
     def _start(self, idle_for, initial_delay=None, stop_on_exception=True):
         """Start the looping
@@ -113,8 +164,8 @@ class LoopingCallBase(object):
         """
         if self._thread is not None:
             raise RuntimeError(self._RUN_ONLY_ONE_MESSAGE)
-        self._running = True
         self.done = event.Event()
+        self._abort.clear()
         self._thread = greenthread.spawn(
             self._run_loop, idle_for,
             initial_delay=initial_delay, stop_on_exception=stop_on_exception)
@@ -128,7 +179,7 @@ class LoopingCallBase(object):
         func = self.f if stop_on_exception else _safe_wrapper(self.f, kind,
                                                               func_name)
         if initial_delay:
-            greenthread.sleep(initial_delay)
+            self._sleep(initial_delay)
         try:
             watch = timeutils.StopWatch()
             while self._running:
@@ -142,13 +193,13 @@ class LoopingCallBase(object):
                           'for %(idle).02f seconds',
                           {'func_name': func_name, 'idle': idle,
                            'kind': kind})
-                greenthread.sleep(idle)
+                self._sleep(idle)
         except LoopingCallDone as e:
             self.done.send(e.retvalue)
         except Exception:
             exc_info = sys.exc_info()
             try:
-                LOG.error(_LE('%(kind)s %(func_name)r failed'),
+                LOG.error('%(kind)s %(func_name)r failed',
                           {'kind': kind, 'func_name': func_name},
                           exc_info=exc_info)
                 self.done.send_exception(*exc_info)
@@ -172,14 +223,45 @@ class FixedIntervalLoopingCall(LoopingCallBase):
             delay = round(elapsed - interval, 2)
             if delay > 0:
                 func_name = reflection.get_callable_name(self.f)
-                LOG.warning(_LW('Function %(func_name)r run outlasted '
-                                'interval by %(delay).2f sec'),
+                LOG.warning('Function %(func_name)r run outlasted '
+                            'interval by %(delay).2f sec',
                             {'func_name': func_name, 'delay': delay})
             return -delay if delay < 0 else 0
         return self._start(_idle_for, initial_delay=initial_delay,
                            stop_on_exception=stop_on_exception)
 
 
+class FixedIntervalWithTimeoutLoopingCall(LoopingCallBase):
+    """A fixed interval looping call with timeout checking mechanism."""
+
+    _RUN_ONLY_ONE_MESSAGE = _("A fixed interval looping call with timeout"
+                              " checking and can only run one function at"
+                              " at a time")
+
+    _KIND = _('Fixed interval looping call with timeout checking.')
+
+    def start(self, interval, initial_delay=None,
+              stop_on_exception=True, timeout=0):
+        start_time = time.time()
+
+        def _idle_for(result, elapsed):
+            delay = round(elapsed - interval, 2)
+            if delay > 0:
+                func_name = reflection.get_callable_name(self.f)
+                LOG.warning('Function %(func_name)r run outlasted '
+                            'interval by %(delay).2f sec',
+                            {'func_name': func_name, 'delay': delay})
+            elapsed_time = time.time() - start_time
+            if timeout > 0 and elapsed_time > timeout:
+                raise LoopingCallTimeOut(
+                    _('Looping call timed out after %.02f seconds')
+                    % elapsed_time)
+            return -delay if delay < 0 else 0
+
+        return self._start(_idle_for, initial_delay=initial_delay,
+                           stop_on_exception=stop_on_exception)
+
+
 class DynamicLoopingCall(LoopingCallBase):
     """A looping call which sleeps until the next known event.
 
@@ -258,6 +340,8 @@ class BackOffLoopingCall(LoopingCallBase):
                    random.gauss(jitter, 0.1), with jitter as the mean for the
                    distribution. If set below .5, it can cause the calls to
                    come more rapidly after each failure.
+    :param min_interval: The minimum interval in seconds between calls to
+                         function.
     :raises: LoopingCallTimeout if time spent doing error retries would exceed
              timeout.
     """
@@ -273,7 +357,7 @@ class BackOffLoopingCall(LoopingCallBase):
         self._interval = 1
 
     def start(self, initial_delay=None, starting_interval=1, timeout=300,
-              max_interval=300, jitter=0.75):
+              max_interval=300, jitter=0.75, min_interval=0.001):
         if self._thread is not None:
             raise RuntimeError(self._RUN_ONLY_ONE_MESSAGE)
 
@@ -282,16 +366,20 @@ class BackOffLoopingCall(LoopingCallBase):
         self._interval = starting_interval
 
         def _idle_for(success, _elapsed):
-            random_jitter = self._RNG.gauss(jitter, 0.1)
+            random_jitter = abs(self._RNG.gauss(jitter, 0.1))
             if success:
                 # Reset error state now that it didn't error...
                 self._interval = starting_interval
                 self._error_time = 0
                 return self._interval * random_jitter
             else:
-                # Perform backoff
-                self._interval = idle = min(
-                    self._interval * 2 * random_jitter, max_interval)
+                # Perform backoff, random jitter around the next interval
+                # bounded by min_interval and max_interval.
+                idle = max(self._interval * 2 * random_jitter, min_interval)
+                idle = min(idle, max_interval)
+                # Calculate the next interval based on the mean, so that the
+                # backoff grows at the desired rate.
+                self._interval = max(self._interval * 2 * jitter, min_interval)
                 # Don't go over timeout, end early if necessary. If
                 # timeout is 0, keep going.
                 if timeout > 0 and self._error_time + idle > timeout:
