@@ -20,7 +20,7 @@ from monotonic import monotonic as now  # noqa
 from oslo_utils import reflection
 import six
 
-from oslo_service._i18n import _, _LE, _LI
+from oslo_service._i18n import _
 from oslo_service import _options
 
 
@@ -106,13 +106,13 @@ class _PeriodicTasksMeta(type):
         name = task._periodic_name
 
         if task._periodic_spacing < 0:
-            LOG.info(_LI('Skipping periodic task %(task)s because '
-                         'its interval is negative'),
+            LOG.info('Skipping periodic task %(task)s because '
+                     'its interval is negative',
                      {'task': name})
             return False
         if not task._periodic_enabled:
-            LOG.info(_LI('Skipping periodic task %(task)s because '
-                         'it is disabled'),
+            LOG.info('Skipping periodic task %(task)s because '
+                     'it is disabled',
                      {'task': name})
             return False
 
@@ -167,7 +167,7 @@ def _nearest_boundary(last_run, spacing):
     delta = current_time - last_run
     offset = delta % spacing
     # Add up to 5% jitter
-    jitter = int(spacing * (random.random() / 20))
+    jitter = int(spacing * (random.random() / 20))  # nosec
     return current_time - offset + jitter
 
 
@@ -218,10 +218,10 @@ class PeriodicTasks(object):
 
             try:
                 task(self, context)
-            except Exception:
+            except BaseException:
                 if raise_on_error:
                     raise
-                LOG.exception(_LE("Error during %(full_task_name)s"),
+                LOG.exception("Error during %(full_task_name)s",
                               {"full_task_name": full_task_name})
             time.sleep(0)
 
