@@ -21,6 +21,9 @@ import abc
 import collections
 import copy
 import errno
+import functools
+import gc
+import inspect
 import io
 import logging
 import os
@@ -34,9 +37,9 @@ import eventlet
 from eventlet import event
 
 from oslo_concurrency import lockutils
-from oslo_service import eventlet_backdoor
-from oslo_service._i18n import _LE, _LI, _LW, _
+from oslo_service._i18n import _
 from oslo_service import _options
+from oslo_service import eventlet_backdoor
 from oslo_service import systemd
 from oslo_service import threadgroup
 
@@ -128,6 +131,9 @@ class SignalHandler(object):
 
     def __init__(self, *args, **kwargs):
         super(SignalHandler, self).__init__(*args, **kwargs)
+
+        self.__setup_signal_interruption()
+
         # Map all signal names to signal integer values and create a
         # reverse mapping (for easier + quick lookup).
         self._ignore_signals = ('SIG_DFL', 'SIG_IGN')
@@ -173,6 +179,66 @@ class SignalHandler(object):
         # clear(): clear() is not reentrant (bug #1538204).
         eventlet.spawn(self._handle_signal_cb, signo, frame)
 
+        # On Python >= 3.5, ensure that eventlet's poll() or sleep() call is
+        # interrupted by raising an exception. If the signal handler does not
+        # raise an exception then due to PEP 475 the call will not return until
+        # an event is detected on a file descriptor or the timeout is reached,
+        # and thus eventlet will not wake up and notice that there has been a
+        # new thread spawned.
+        if self.__force_interrupt_on_signal:
+            try:
+                interrupted_frame = inspect.stack(context=0)[1]
+            except IndexError:
+                pass
+            else:
+                if ((interrupted_frame.function == 'do_poll' and
+                     interrupted_frame.filename == self.__hub_module_file) or
+                    (interrupted_frame.function == 'do_sleep' and
+                     interrupted_frame.filename == __file__)):
+                    raise IOError(errno.EINTR, 'Interrupted')
+
+    def __setup_signal_interruption(self):
+        """Set up to do the Right Thing with signals during poll() and sleep().
+
+        For Python 3.5 and later, deal with the changes in PEP 475 that prevent
+        a signal from interrupting eventlet's call to poll() or sleep().
+        """
+        select_module = eventlet.patcher.original('select')
+        self.__force_interrupt_on_signal = (sys.version_info >= (3, 5) and
+                                            hasattr(select_module, 'poll'))
+
+        if self.__force_interrupt_on_signal:
+            try:
+                from eventlet.hubs import poll as poll_hub
+            except ImportError:
+                pass
+            else:
+                # This is a function we can test for in the stack when handling
+                # a signal - it's safe to raise an IOError with EINTR anywhere
+                # in this function.
+                def do_sleep(time_sleep_func, seconds):
+                    return time_sleep_func(seconds)
+
+                time_sleep = eventlet.patcher.original('time').sleep
+
+                # Wrap time.sleep to ignore the interruption error we're
+                # injecting from the signal handler. This makes the behaviour
+                # the same as sleep() in Python 2, where EINTR causes the
+                # sleep to be interrupted (and not resumed), but no exception
+                # is raised.
+                @functools.wraps(time_sleep)
+                def sleep_wrapper(seconds):
+                    try:
+                        return do_sleep(time_sleep, seconds)
+                    except (IOError, InterruptedError) as err:
+                        if err.errno != errno.EINTR:
+                            raise
+
+                poll_hub.sleep = sleep_wrapper
+
+            hub = eventlet.hubs.get_hub()
+            self.__hub_module_file = sys.modules[hub.__module__].__file__
+
     def _handle_signal_cb(self, signo, frame):
         for handler in self._signal_handlers[signo]:
             handler(signo, frame)
@@ -239,8 +305,7 @@ class Launcher(object):
         """Reload config files and restart service.
 
         :returns: The return value from reload_config_files or
-        mutate_config_files, according to the restart_method.
-
+          mutate_config_files, according to the restart_method.
         """
         if self.restart_method == 'reload':
             self.conf.reload_config_files()
@@ -279,16 +344,17 @@ class ServiceLauncher(Launcher):
         raise SignalExit(signal.SIGHUP)
 
     def _fast_exit(self, *args):
-        LOG.info(_LI('Caught SIGINT signal, instantaneous exiting'))
+        LOG.info('Caught SIGINT signal, instantaneous exiting')
         os._exit(1)
 
     def _on_timeout_exit(self, *args):
-        LOG.info(_LI('Graceful shutdown timeout exceeded, '
-                     'instantaneous exiting'))
+        LOG.info('Graceful shutdown timeout exceeded, '
+                 'instantaneous exiting')
         os._exit(1)
 
     def handle_signal(self):
         """Set self._handle_signal as a signal handler."""
+        self.signal_handler.clear()
         self.signal_handler.add_handler('SIGTERM', self._graceful_shutdown)
         self.signal_handler.add_handler('SIGINT', self._fast_exit)
         self.signal_handler.add_handler('SIGHUP', self._reload_service)
@@ -306,7 +372,7 @@ class ServiceLauncher(Launcher):
             super(ServiceLauncher, self).wait()
         except SignalExit as exc:
             signame = self.signal_handler.signals_to_name[exc.signo]
-            LOG.info(_LI('Caught %s, exiting'), signame)
+            LOG.info('Caught %s, exiting', signame)
             status = exc.code
             signo = exc.signo
         except SystemExit as exc:
@@ -372,13 +438,13 @@ class ProcessLauncher(object):
 
     def handle_signal(self):
         """Add instance's signal handlers to class handlers."""
-        self.signal_handler.add_handlers(('SIGTERM', 'SIGHUP'),
-                                         self._handle_signal)
+        self.signal_handler.add_handler('SIGTERM', self._handle_term)
+        self.signal_handler.add_handler('SIGHUP', self._handle_hup)
         self.signal_handler.add_handler('SIGINT', self._fast_exit)
         self.signal_handler.add_handler('SIGALRM', self._on_alarm_exit)
 
-    def _handle_signal(self, signo, frame):
-        """Set signal handlers.
+    def _handle_term(self, signo, frame):
+        """Handle a TERM event.
 
         :param signo: signal number
         :param frame: current stack frame
@@ -389,13 +455,26 @@ class ProcessLauncher(object):
         # Allow the process to be killed again and die from natural causes
         self.signal_handler.clear()
 
+    def _handle_hup(self, signo, frame):
+        """Handle a HUP event.
+
+        :param signo: signal number
+        :param frame: current stack frame
+        """
+        self.sigcaught = signo
+        self.running = False
+
+        # Do NOT clear the signal_handler, allowing multiple SIGHUPs to be
+        # received swiftly. If a non-HUP is received before #wait loops, the
+        # second event will "overwrite" the HUP. This is fine.
+
     def _fast_exit(self, signo, frame):
-        LOG.info(_LI('Caught SIGINT signal, instantaneous exiting'))
+        LOG.info('Caught SIGINT signal, instantaneous exiting')
         os._exit(1)
 
     def _on_alarm_exit(self, signo, frame):
-        LOG.info(_LI('Graceful shutdown timeout exceeded, '
-                     'instantaneous exiting'))
+        LOG.info('Graceful shutdown timeout exceeded, '
+                 'instantaneous exiting')
         os._exit(1)
 
     def _pipe_watcher(self):
@@ -403,7 +482,7 @@ class ProcessLauncher(object):
         # dies unexpectedly
         self.readpipe.read(1)
 
-        LOG.info(_LI('Parent process has died unexpectedly, exiting'))
+        LOG.info('Parent process has died unexpectedly, exiting')
 
         if self.launcher:
             self.launcher.stop()
@@ -439,13 +518,15 @@ class ProcessLauncher(object):
             launcher.wait()
         except SignalExit as exc:
             signame = self.signal_handler.signals_to_name[exc.signo]
-            LOG.info(_LI('Child caught %s, exiting'), signame)
+            LOG.info('Child caught %s, exiting', signame)
             status = exc.code
             signo = exc.signo
         except SystemExit as exc:
+            launcher.stop()
             status = exc.code
         except BaseException:
-            LOG.exception(_LE('Unhandled exception'))
+            launcher.stop()
+            LOG.exception('Unhandled exception')
             status = 2
 
         return status, signo
@@ -476,7 +557,7 @@ class ProcessLauncher(object):
             # start up quickly but ensure we don't fork off children that
             # die instantly too quickly.
             if time.time() - wrap.forktimes[0] < wrap.workers:
-                LOG.info(_LI('Forking too fast, sleeping'))
+                LOG.info('Forking too fast, sleeping')
                 time.sleep(1)
 
             wrap.forktimes.pop(0)
@@ -515,7 +596,14 @@ class ProcessLauncher(object):
         _check_service_base(service)
         wrap = ServiceWrapper(service, workers)
 
-        LOG.info(_LI('Starting %d workers'), wrap.workers)
+        # Hide existing objects from the garbage collector, so that most
+        # existing pages will remain in shared memory rather than being
+        # duplicated between subprocesses in the GC mark-and-sweep. (Requires
+        # Python 3.7 or later.)
+        if hasattr(gc, 'freeze'):
+            gc.freeze()
+
+        LOG.info('Starting %d workers', wrap.workers)
         while self.running and len(wrap.children) < wrap.workers:
             self._start_child(wrap)
 
@@ -532,15 +620,15 @@ class ProcessLauncher(object):
 
         if os.WIFSIGNALED(status):
             sig = os.WTERMSIG(status)
-            LOG.info(_LI('Child %(pid)d killed by signal %(sig)d'),
+            LOG.info('Child %(pid)d killed by signal %(sig)d',
                      dict(pid=pid, sig=sig))
         else:
             code = os.WEXITSTATUS(status)
-            LOG.info(_LI('Child %(pid)s exited with status %(code)d'),
+            LOG.info('Child %(pid)s exited with status %(code)d',
                      dict(pid=pid, code=code))
 
         if pid not in self.children:
-            LOG.warning(_LW('pid %d not in child list'), pid)
+            LOG.warning('pid %d not in child list', pid)
             return None
 
         wrap = self.children.pop(pid)
@@ -576,7 +664,7 @@ class ProcessLauncher(object):
                     return
 
                 signame = self.signal_handler.signals_to_name[self.sigcaught]
-                LOG.info(_LI('Caught %s, stopping children'), signame)
+                LOG.info('Caught %s, stopping children', signame)
                 if not _is_sighup_and_daemon(self.sigcaught):
                     break
 
@@ -594,7 +682,7 @@ class ProcessLauncher(object):
                 self.running = True
                 self.sigcaught = None
         except eventlet.greenlet.GreenletExit:
-            LOG.info(_LI("Wait called after thread killed. Cleaning up."))
+            LOG.info("Wait called after thread killed. Cleaning up.")
 
         # if we are here it means that we are trying to do graceful shutdown.
         # add alarm watching that graceful_shutdown_timeout is not exceeded
@@ -623,7 +711,7 @@ class ProcessLauncher(object):
 
         # Wait for children to die
         if self.children:
-            LOG.info(_LI('Waiting on %d children to exit'), len(self.children))
+            LOG.info('Waiting on %d children to exit', len(self.children))
             while self.children:
                 self._wait_child()
 
@@ -707,7 +795,7 @@ class Services(object):
         try:
             service.start()
         except Exception:
-            LOG.exception(_LE('Error starting thread.'))
+            LOG.exception('Error starting thread.')
             raise SystemExit(1)
         else:
             done.wait()
