@@ -20,9 +20,6 @@ from oslotest import base as test_base
 import oslo_service
 from oslo_service import loopingcall
 
-threading = eventlet.patcher.original('threading')
-time = eventlet.patcher.original('time')
-
 
 class LoopingCallTestCase(test_base.BaseTestCase):
 
@@ -53,45 +50,6 @@ class LoopingCallTestCase(test_base.BaseTestCase):
         self.assertEqual(oslo_service._monotonic,
                          hub.clock)
 
-    def test_eventlet_use_hub_override(self):
-        ns = {}
-
-        def task():
-            try:
-                self._test_eventlet_use_hub_override()
-            except Exception as exc:
-                ns['result'] = exc
-            else:
-                ns['result'] = 'ok'
-
-        # test overriding the hub of in a new thread to not modify the hub
-        # of the main thread
-        thread = threading.Thread(target=task)
-        thread.start()
-        thread.join()
-        self.assertEqual('ok', ns['result'])
-
-    def _test_eventlet_use_hub_override(self):
-        # Make sure that by default the
-        # oslo_service.service_hub() kicks in
-        old_clock = eventlet.hubs.get_hub().clock
-        self.assertEqual(oslo_service._monotonic,
-                         old_clock)
-
-        # eventlet will use time.monotonic() by default, same clock than
-        # oslo.service_hub():
-        # https://github.com/eventlet/eventlet/pull/303
-        if not hasattr(time, 'monotonic'):
-            # If anyone wants to override it
-            try:
-                eventlet.hubs.use_hub('poll')
-            except Exception:
-                eventlet.hubs.use_hub('selects')
-
-            # then we get a new clock and the override works fine too!
-            clock = eventlet.hubs.get_hub().clock
-            self.assertNotEqual(old_clock, clock)
-
     def test_return_false(self):
         def _raise_it():
             raise loopingcall.LoopingCallDone(False)
@@ -142,6 +100,16 @@ class LoopingCallTestCase(test_base.BaseTestCase):
         wait_ev.set()
         timer.wait()
 
+    def test_no_double_stop(self):
+        def _raise_it():
+            raise loopingcall.LoopingCallDone(False)
+
+        timer = loopingcall.FixedIntervalLoopingCall(_raise_it)
+        timer.start(interval=0.5)
+
+        timer.stop()
+        timer.stop()
+
     def test_repeat(self):
         self.num_runs = 2
 
@@ -180,6 +148,15 @@ class LoopingCallTestCase(test_base.BaseTestCase):
                        (i, expected, actual))
             self.assertAlmostEqual(expected, actual, message=message)
 
+    def test_looping_call_timed_out(self):
+
+        def _fake_task():
+            pass
+
+        timer = loopingcall.FixedIntervalWithTimeoutLoopingCall(_fake_task)
+        self.assertRaises(loopingcall.LoopingCallTimeOut,
+                          timer.start(interval=0.1, timeout=0.3).wait)
+
 
 class DynamicLoopingCallTestCase(test_base.BaseTestCase):
     def setUp(self):
@@ -276,7 +253,7 @@ class DynamicLoopingCallTestCase(test_base.BaseTestCase):
         else:
             self.num_runs = self.num_runs - 1
 
-    @mock.patch('eventlet.greenthread.sleep')
+    @mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep')
     def test_timeout_task_without_return(self, sleep_mock):
         self.num_runs = 1
         timer = loopingcall.DynamicLoopingCall(
@@ -285,7 +262,7 @@ class DynamicLoopingCallTestCase(test_base.BaseTestCase):
         timer.start(periodic_interval_max=5).wait()
         sleep_mock.assert_has_calls([mock.call(5)])
 
-    @mock.patch('eventlet.greenthread.sleep')
+    @mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep')
     def test_interval_adjustment(self, sleep_mock):
         self.num_runs = 2
 
@@ -294,7 +271,7 @@ class DynamicLoopingCallTestCase(test_base.BaseTestCase):
 
         sleep_mock.assert_has_calls([mock.call(5), mock.call(1)])
 
-    @mock.patch('eventlet.greenthread.sleep')
+    @mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep')
     def test_initial_delay(self, sleep_mock):
         self.num_runs = 1
 
@@ -306,7 +283,7 @@ class DynamicLoopingCallTestCase(test_base.BaseTestCase):
 
 class TestBackOffLoopingCall(test_base.BaseTestCase):
     @mock.patch('random.SystemRandom.gauss')
-    @mock.patch('eventlet.greenthread.sleep')
+    @mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep')
     def test_exponential_backoff(self, sleep_mock, random_mock):
         def false():
             return False
@@ -317,20 +294,47 @@ class TestBackOffLoopingCall(test_base.BaseTestCase):
                           loopingcall.BackOffLoopingCall(false).start()
                           .wait)
 
-        expected_times = [mock.call(1.6000000000000001),
-                          mock.call(2.5600000000000005),
-                          mock.call(4.096000000000001),
-                          mock.call(6.5536000000000021),
-                          mock.call(10.485760000000004),
-                          mock.call(16.777216000000006),
-                          mock.call(26.843545600000013),
-                          mock.call(42.949672960000022),
-                          mock.call(68.719476736000033),
-                          mock.call(109.95116277760006)]
+        expected_times = [mock.call(1.6),
+                          mock.call(2.4000000000000004),
+                          mock.call(3.6),
+                          mock.call(5.4),
+                          mock.call(8.1),
+                          mock.call(12.15),
+                          mock.call(18.225),
+                          mock.call(27.337500000000002),
+                          mock.call(41.00625),
+                          mock.call(61.509375000000006),
+                          mock.call(92.26406250000001)]
         self.assertEqual(expected_times, sleep_mock.call_args_list)
 
     @mock.patch('random.SystemRandom.gauss')
-    @mock.patch('eventlet.greenthread.sleep')
+    @mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep')
+    def test_exponential_backoff_negative_value(self, sleep_mock, random_mock):
+        def false():
+            return False
+
+        # random.gauss() can return negative values
+        random_mock.return_value = -.8
+
+        self.assertRaises(loopingcall.LoopingCallTimeOut,
+                          loopingcall.BackOffLoopingCall(false).start()
+                          .wait)
+
+        expected_times = [mock.call(1.6),
+                          mock.call(2.4000000000000004),
+                          mock.call(3.6),
+                          mock.call(5.4),
+                          mock.call(8.1),
+                          mock.call(12.15),
+                          mock.call(18.225),
+                          mock.call(27.337500000000002),
+                          mock.call(41.00625),
+                          mock.call(61.509375000000006),
+                          mock.call(92.26406250000001)]
+        self.assertEqual(expected_times, sleep_mock.call_args_list)
+
+    @mock.patch('random.SystemRandom.gauss')
+    @mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep')
     def test_no_backoff(self, sleep_mock, random_mock):
         random_mock.return_value = 1
         func = mock.Mock()
@@ -345,7 +349,7 @@ class TestBackOffLoopingCall(test_base.BaseTestCase):
         self.assertTrue(retvalue, 'return value')
 
     @mock.patch('random.SystemRandom.gauss')
-    @mock.patch('eventlet.greenthread.sleep')
+    @mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep')
     def test_no_sleep(self, sleep_mock, random_mock):
         # Any call that executes properly the first time shouldn't sleep
         random_mock.return_value = 1
@@ -358,7 +362,7 @@ class TestBackOffLoopingCall(test_base.BaseTestCase):
         self.assertTrue(retvalue, 'return value')
 
     @mock.patch('random.SystemRandom.gauss')
-    @mock.patch('eventlet.greenthread.sleep')
+    @mock.patch('oslo_service.loopingcall.LoopingCallBase._sleep')
     def test_max_interval(self, sleep_mock, random_mock):
         def false():
             return False
@@ -370,14 +374,15 @@ class TestBackOffLoopingCall(test_base.BaseTestCase):
                               max_interval=60)
                           .wait)
 
-        expected_times = [mock.call(1.6000000000000001),
-                          mock.call(2.5600000000000005),
-                          mock.call(4.096000000000001),
-                          mock.call(6.5536000000000021),
-                          mock.call(10.485760000000004),
-                          mock.call(16.777216000000006),
-                          mock.call(26.843545600000013),
-                          mock.call(42.949672960000022),
+        expected_times = [mock.call(1.6),
+                          mock.call(2.4000000000000004),
+                          mock.call(3.6),
+                          mock.call(5.4),
+                          mock.call(8.1),
+                          mock.call(12.15),
+                          mock.call(18.225),
+                          mock.call(27.337500000000002),
+                          mock.call(41.00625),
                           mock.call(60),
                           mock.call(60),
                           mock.call(60)]
