@@ -454,9 +454,10 @@ class ProcessLauncherTest(base.ServiceBaseTestCase):
 
         self.assertFalse(launcher.running)
         self.assertFalse(launcher.children)
-        self.assertEqual([mock.call(222, signal_mock.SIGTERM),
-                          mock.call(22, signal_mock.SIGTERM)],
-                         mock_kill.mock_calls)
+        mock_kill.assert_has_calls([mock.call(222, signal_mock.SIGTERM),
+                                    mock.call(22, signal_mock.SIGTERM)],
+                                   any_order=True)
+        self.assertEqual(2, mock_kill.call_count)
         mock_service_stop.assert_called_once_with()
 
     def test__handle_signal(self):
@@ -476,6 +477,31 @@ class ProcessLauncherTest(base.ServiceBaseTestCase):
             m.assert_called_once_with(signal.SIGTERM, 'test')
         signal_handler.clear()
 
+    @mock.patch('sys.version_info', (3, 5))
+    def test_setup_signal_interruption_no_select_poll(self):
+        # NOTE(claudiub): SignalHandler is a singleton, which means that it
+        # might already be initialized. We need to clear to clear the cache
+        # in order to prevent race conditions between tests.
+        service.SignalHandler.__class__._instances.clear()
+        with mock.patch('eventlet.patcher.original',
+                        return_value=object()) as get_original:
+            signal_handler = service.SignalHandler()
+            get_original.assert_called_with('select')
+        self.addCleanup(service.SignalHandler.__class__._instances.clear)
+        self.assertFalse(
+            signal_handler._SignalHandler__force_interrupt_on_signal)
+
+    @mock.patch('sys.version_info', (3, 5))
+    def test_setup_signal_interruption_select_poll(self):
+        # NOTE(claudiub): SignalHandler is a singleton, which means that it
+        # might already be initialized. We need to clear to clear the cache
+        # in order to prevent race conditions between tests.
+        service.SignalHandler.__class__._instances.clear()
+        signal_handler = service.SignalHandler()
+        self.addCleanup(service.SignalHandler.__class__._instances.clear)
+        self.assertTrue(
+            signal_handler._SignalHandler__force_interrupt_on_signal)
+
     @mock.patch('signal.alarm')
     @mock.patch("os.kill")
     @mock.patch("oslo_service.service.ProcessLauncher.stop")
@@ -535,6 +561,38 @@ class ProcessLauncherTest(base.ServiceBaseTestCase):
         serv = FooService()
         self.assertRaises(TypeError, launcher.launch_service, serv, 0)
 
+    @mock.patch("oslo_service.service.ProcessLauncher._start_child")
+    @mock.patch("oslo_service.service.ProcessLauncher.handle_signal")
+    @mock.patch("eventlet.greenio.GreenPipe")
+    @mock.patch("os.pipe")
+    def test_double_sighup(self, pipe_mock, green_pipe_mock,
+                           handle_signal_mock, start_child_mock):
+        # Test that issuing two SIGHUPs in a row does not exit; then send a
+        # TERM that does cause an exit.
+        pipe_mock.return_value = [None, None]
+        launcher = service.ProcessLauncher(self.conf)
+        serv = _Service()
+        launcher.launch_service(serv, workers=0)
+
+        def stager():
+            # -1: start state
+            # 0: post-init
+            # 1: first HUP sent
+            # 2: second HUP sent
+            # 3: TERM sent
+            stager.stage += 1
+            if stager.stage < 3:
+                launcher._handle_hup(1, mock.sentinel.frame)
+            elif stager.stage == 3:
+                launcher._handle_term(15, mock.sentinel.frame)
+            else:
+                self.fail("TERM did not kill launcher")
+        stager.stage = -1
+        handle_signal_mock.side_effect = stager
+
+        launcher.wait()
+        self.assertEqual(3, stager.stage)
+
 
 class GracefulShutdownTestService(service.Service):
     def __init__(self):
