@@ -11,15 +11,15 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
 import logging
 import threading
 
 import eventlet
 from eventlet import greenpool
 
-from oslo_service._i18n import _LE
 from oslo_service import loopingcall
-
+from oslo_utils import timeutils
 
 LOG = logging.getLogger(__name__)
 
@@ -40,9 +40,10 @@ class Thread(object):
      the :class:`ThreadGroup` when it has done so it can be removed from
      the threads list.
     """
-    def __init__(self, thread, group):
+    def __init__(self, thread, group, link=True):
         self.thread = thread
-        self.thread.link(_on_thread_done, group, self)
+        if link:
+            self.thread.link(_on_thread_done, group, self)
         self._ident = id(thread)
 
     @property
@@ -58,6 +59,9 @@ class Thread(object):
     def link(self, func, *args, **kwargs):
         self.thread.link(func, *args, **kwargs)
 
+    def cancel(self, *throw_args):
+        self.thread.cancel(*throw_args)
+
 
 class ThreadGroup(object):
     """The point of the ThreadGroup class is to:
@@ -89,8 +93,9 @@ class ThreadGroup(object):
 
     def add_thread(self, callback, *args, **kwargs):
         gt = self.pool.spawn(callback, *args, **kwargs)
-        th = Thread(gt, self)
+        th = Thread(gt, self, link=False)
         self.threads.append(th)
+        gt.link(_on_thread_done, self, th)
         return th
 
     def thread_done(self, thread):
@@ -118,11 +123,13 @@ class ThreadGroup(object):
     def _stop_threads(self):
         self._perform_action_on_threads(
             lambda x: x.stop(),
-            lambda x: LOG.exception(_LE('Error stopping thread.')))
+            lambda x: LOG.exception('Error stopping thread.'))
 
-    def stop_timers(self):
+    def stop_timers(self, wait=False):
         for timer in self.timers:
             timer.stop()
+        if wait:
+            self._wait_timers()
         self.timers = []
 
     def stop(self, graceful=False):
@@ -132,17 +139,17 @@ class ThreadGroup(object):
           Never kill threads.
         * In case of graceful=False, kill threads immediately.
         """
-        self.stop_timers()
+        self.stop_timers(wait=graceful)
         if graceful:
             # In case of graceful=True, wait for all threads to be
             # finished, never kill threads
-            self.wait()
+            self._wait_threads()
         else:
             # In case of graceful=False(Default), kill threads
             # immediately
             self._stop_threads()
 
-    def wait(self):
+    def _wait_timers(self):
         for x in self.timers:
             try:
                 x.wait()
@@ -150,7 +157,41 @@ class ThreadGroup(object):
                 # greenlet exited successfully
                 pass
             except Exception:
-                LOG.exception(_LE('Error waiting on timer.'))
+                LOG.exception('Error waiting on timer.')
+
+    def _wait_threads(self):
         self._perform_action_on_threads(
             lambda x: x.wait(),
-            lambda x: LOG.exception(_LE('Error waiting on thread.')))
+            lambda x: LOG.exception('Error waiting on thread.'))
+
+    def wait(self):
+        self._wait_timers()
+        self._wait_threads()
+
+    def _any_threads_alive(self):
+        current = threading.current_thread()
+        for x in self.threads[:]:
+            if x.ident == current.ident:
+                # Don't check current thread.
+                continue
+            if not x.thread.dead:
+                return True
+        return False
+
+    def cancel(self, *throw_args, **kwargs):
+        self._perform_action_on_threads(
+            lambda x: x.cancel(*throw_args),
+            lambda x: LOG.exception('Error canceling thread.'))
+
+        timeout = kwargs.get('timeout', None)
+        if timeout is None:
+            return
+        wait_time = kwargs.get('wait_time', 1)
+        watch = timeutils.StopWatch(duration=timeout)
+        watch.start()
+        while self._any_threads_alive():
+            if not watch.expired():
+                eventlet.sleep(wait_time)
+                continue
+            LOG.debug("Cancel timeout reached, stopping threads.")
+            self.stop()
